@@ -1,17 +1,21 @@
 import os
+from typing import Callable
+from typing import Optional as O
+from typing import Sequence
+from typing import Union as U
 
 from ..base import data_readers, df_creators, df_manipulators
 from ..base.histo_dataset import HistoDataset
 
 
 def mai_patches(
-    root,
-    transformation=None,
-    pre_transformation=None,
-    folds=None,
-    gt_mode="GT3&GT2",
-    stain_mode="phh3",
-    seed=None,
+    root: str,
+    transformation: O[U[Callable, Sequence[Callable]]] = None,
+    pre_transformation: O[U[Callable, Sequence[Callable]]] = None,
+    folds: O[Sequence[int]] = None,
+    gt_mode: str = "GT3&GT2",
+    stain_mode: str = "phh3",
+    seed: O[int] = None,
 ):
     r"""
     The functions returns a pytorch dataset to load the MAI patches. The
@@ -54,7 +58,26 @@ def mai_patches(
                                 none-tumor regionen
                        - "without": no label will be returned.
         stain_mode (str): The stain to return, one of "phh3", "he", "he&phh3"
-        seed (int): The seed that is used for the pre_transformation.
+        seed (int): The seed that is used for the "pre_transformation" and the
+                    "transformation". If the seed is set, the "pre_transformation"
+                    will always use the same seed for a data row. If the seed is
+                    set, the "transformation" will use always the same seed
+                    depending on the call position. For example:
+                    Call-Index,     Seed f.              Seed f.
+                                pre_transformation,  transformation
+                        0     ,     100           ,    124
+                        1     ,     100           ,    512
+                        2     ,     100           ,    810
+                        3     ,     100           ,    612
+                    If we create the exact same dataset the transformation seeds
+                    will be the same:
+                    Call-Index,     Seed f.              Seed f.
+                                pre_transformation,  transformation
+                        0     ,     100           ,    124
+                        1     ,     100           ,    512
+                        2     ,     100           ,    810
+                        3     ,     100           ,    612
+
 
     Returns:
         HistoDataset: The dataset that loads the mai patches. You can use this
@@ -145,14 +168,85 @@ def mai_patches(
     return ds
 
 
+def mai_patches_bern(
+    root: str,
+    transformation: O[U[Callable, Sequence[Callable]]] = None,
+    pre_transformation: O[U[Callable, Sequence[Callable]]] = None,
+    stain_mode: str = "phh3",
+    seed: O[int] = None,
+):
+    r"""
+    The functions returns a pytorch dataset to load the MAI patches
+    that was created from bern. There exists no labels to this dataset.
+
+    Args:
+        root (str): The path to the dataset.
+        transformation (Callable): A callable or a list of
+                       callable transformations.
+        pre_transformation (Callable): A callable or a list of
+                       callable transformations. "pre_transformation"
+                       is called before "transformation". The transformation
+                       is called with a seed and will give always the same
+                       results.
+        stain_mode (str): currently only the phh3 stain exists.
+        seed (int): The seed that is used for the "pre_transformation" and the
+                    "transformation". If the seed is set, the "pre_transformation"
+                    will always use the same seed for a data row. If the seed is
+                    set, the "transformation" will use always the same seed
+                    depending on the call position. For example:
+                    Call-Index,     Seed f.              Seed f.
+                                pre_transformation,  transformation
+                        0     ,     100           ,    124
+                        1     ,     100           ,    512
+                        2     ,     100           ,    810
+                        3     ,     100           ,    612
+                    If we create the exact same dataset the transformation seeds
+                    will be the same:
+                    Call-Index,     Seed f.              Seed f.
+                                pre_transformation,  transformation
+                        0     ,     100           ,    124
+                        1     ,     100           ,    512
+                        2     ,     100           ,    810
+                        3     ,     100           ,    612
+
+    Returns:
+        HistoDataset: The dataset that loads the mai patches. You can use this
+                      as normal pytorch dataset. If you call it you will get
+                      a dictionary back.
+    """
+    if stain_mode == "phh3":
+        readers_to_use = "PHH3/{base}.mrxs_{x}_{y}.png"
+    elif stain_mode == "he_just_a_tmp_do_not_use_this":
+        readers_to_use = "HE/{base}.mrxs_{x}_{y}.png"
+    elif stain_mode == "he&phh3_just_a_tmp_do_not_use_this":
+        readers_to_use = ["HE/{base}.mrxs_{x}_{y}.png", "PHH3/{base}.mrxs_{x}_{y}.png"]
+    else:
+        raise NotImplementedError(
+            "The stain_mode must be one of the following values: "
+            + '["phh3"]. The default value is phh3'
+        )
+
+    ds = HistoDataset(
+        df_creators.CreateDFFromFolder("PHH3/{base}.mrxs_{x}_{y}.png"),
+        root,
+        df_manipulators=None,
+        data_readers=readers_to_use,
+        feature_readers=None,
+        seed=seed,
+        pre_transfs=pre_transformation,
+        da_transfs=transformation,
+    )
+    return ds
+
+
 def mai_he_and_phh3_fields(
-    root,
-    split,
-    splitting_seed=69,
-    transformation=None,
-    pre_transformation=None,
-    patch_size=224,
-    overlapping_percentage=0.0,
+    root: str,
+    split: str,
+    splitting_seed: O[int] = 69,
+    transformation: O[U[Callable, Sequence[Callable]]] = None,
+    pre_transformation: O[U[Callable, Sequence[Callable]]] = None,
+    patch_size: O[int] = 224,
+    overlapping_percentage: O[float] = 0.0,
     seed=None,
     stain_mode="he&phh3",
     splitting_trainings_percentage=0.8,
@@ -187,7 +281,25 @@ def mai_he_and_phh3_fields(
                        is used for the overlapping. The value 0.5 means that
                        50 percent of one patch to the next patch are the same.
         stain_mode (str): The stain to return, one of "phh3", "he", "he&phh3"
-        seed (int): The seed that is used for the pre_transformation.
+        seed (int): The seed that is used for the "pre_transformation" and the
+                    "transformation". If the seed is set, the "pre_transformation"
+                    will always use the same seed for a data row. If the seed is
+                    set, the "transformation" will use always the same seed
+                    depending on the call position. For example:
+                    Call-Index,     Seed f.              Seed f.
+                                pre_transformation,  transformation
+                        0     ,     100           ,    124
+                        1     ,     100           ,    512
+                        2     ,     100           ,    810
+                        3     ,     100           ,    612
+                    If we create the exact same dataset the transformation seeds
+                    will be the same:
+                    Call-Index,     Seed f.              Seed f.
+                                pre_transformation,  transformation
+                        0     ,     100           ,    124
+                        1     ,     100           ,    512
+                        2     ,     100           ,    810
+                        3     ,     100           ,    612
 
     Returns:
         HistoDataset: The dataset that loads the mai patches. You can use this
